@@ -1,3 +1,10 @@
+//! DMS Coordinates
+//!
+//! Package to manipulate D°M'S'' coordinates
+//! mainly in navigation applications.
+//!
+//! Homepage: <https://github.com/gwbres/dms-coordinates>
+
 /// List of known bearings to construct a `D°M'S''`
 pub const KNOWN_BEARINGS: &'static [char] = &['N','S','E','W'];
 
@@ -17,6 +24,23 @@ macro_rules! single_line_if_else {
     };
 }
 
+const EARTH_RADIUS: f64 = 6371E3_f64;
+
+/// Converts x (f64) to radians
+fn deg2rad (x: f64) -> f64 { x/180.0*std::f64::consts::PI }
+
+/// Returns distance (m) between two decimal degrees coordinates::
+/// coord1: (lat,lon), coord2: (lat, lon)
+fn distance (coord1: (f64,f64), coord2: (f64,f64)) -> f64 {
+    let dphi = deg2rad(coord2.0) - deg2rad(coord1.0);
+    let d_lambda = deg2rad(coord2.1) - deg2rad(coord1.1);
+    let a: f64 = (dphi / 2.0_f64).sin().powf(2.0_f64)
+        + deg2rad(coord1.0).cos() * deg2rad(coord2.0).cos()
+            * (d_lambda/2.0_f64).sin().powf(2.0_f64);
+    let c = 2.0_f64 * a.powf(0.5_f64).atan2((1.0-a).powf(0.5_f64));
+    EARTH_RADIUS * c
+}
+
 impl DMS {
     /// Builds new `D°M'S''` coordinates
     pub fn new (degrees: i32, minutes: i32, seconds: f64, bearing: char) -> Result<DMS, std::io::Error> {
@@ -32,9 +56,7 @@ impl DMS {
     /// longitude otherwise.
     pub fn from_decimal_degrees (ddeg: f64, is_lat: bool) -> DMS {
         let d = ddeg.abs().trunc() as i32;
-        //let m = ((ddeg - d as f64) * 60.0).trunc() as i32;
         let m = ((ddeg.abs() - d as f64) * 60.0).trunc() as i32;
-        //let s = ((ddeg - d as f64 - (m as f64)/60.0_f64)) * 3600_f64;
         let s = (ddeg.abs() - d as f64 - (m as f64)/60.0_f64) * 3600.0_f64;
 
         let bearing = match is_lat {
@@ -50,16 +72,16 @@ impl DMS {
         }
     }
 
-    // Returns D°
+    /// Returns D°
     pub fn get_degrees (&self) -> i32 { return self.degrees }
-    // Returns M'
+    /// Returns M'
     pub fn get_minutes (&self) -> i32 { return self.minutes }
-    // Returns S''
+    /// Returns S''
     pub fn get_seconds (&self) -> f64 { return self.seconds }
-    // Returns bearing 
+    /// Returns bearing 
     pub fn get_bearing (&self) -> char { return self.bearing }
 
-    // Converts `D°M'S''` coordinates to `Decimal Degrees` coordinates
+    /// Converts `D°M'S''` coordinates to `Decimal Degrees` coordinates
     pub fn to_decimal_degrees (&self) -> f64 {
         let ddeg: f64 = self.degrees as f64
             + self.minutes as f64 / 60.0_f64
@@ -102,8 +124,15 @@ impl DMS3d {
 
     /// Returns altitude of self
     pub fn get_altitude (&self) -> Option<f64> { self.altitude }
-}
 
+    /// Returns distance (m) between self and another DMS3d
+    pub fn distance (&self, other: DMS3d) -> f64 {
+        distance(
+            (self.latitude.to_decimal_degrees(),self.longitude.to_decimal_degrees()),
+            (other.latitude.to_decimal_degrees(),other.longitude.to_decimal_degrees())
+        )
+    }
+}
 
 #[cfg(test)]
 mod tests {
@@ -185,4 +214,21 @@ mod tests {
         assert_eq!(dms.longitude.get_bearing(), 'W');
         assert!((dms.longitude.get_seconds() - 6.8712).abs() < 1E-3);
     }
+
+    #[test]
+    fn test_distance() {
+        let dms1 = DMS3d::from_decimal_degrees( // NY
+            40.730610_f64,
+            -73.935242_f64,
+            Some(10.0)
+        );
+        let dms2 = DMS3d::from_decimal_degrees( // Paris
+            48.856614, 
+            2.3522219,
+            Some(10.0)
+        );
+        let expected_km = 5831.0_f64; 
+        let d_km = dms1.distance(dms2) / 1000.0_f64;
+        assert!((expected_km - d_km).abs() < 1.0);
+    }
 }
